#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import implementer
from zope.schema import Object
from zope.schema.interfaces import IObject

from pyams_gis.interfaces import IGeoArea, IGeoPoint, IGeoPointZ

__docformat__ = 'restructuredtext'


class IGeoPointField(IObject):
    """GeoPoint schema field interface"""


@implementer(IGeoPointField)
class GeoPointField(Object):
    """GeoPoint field class"""

    def __init__(self, **kwargs):
        if 'schema' in kwargs:
            del kwargs['schema']
        super(GeoPointField, self).__init__(IGeoPoint, **kwargs)


class IGeoPointZField(IObject):
    """GeoPointZ schema field interface"""


@implementer(IGeoPointZField)
class GeoPointZField(Object):
    """GeoPointZ field class"""

    def __init__(self, **kwargs):
        if 'schema' in kwargs:
            del kwargs['schema']
        super(GeoPointZField, self).__init__(IGeoPointZ, **kwargs)


class IGeoAreaField(IObject):
    """GeoArea schema field interface"""


@implementer(IGeoAreaField)
class GeoAreaField(Object):
    """GeoArea field class"""

    def __init__(self, **kwargs):
        if 'schema' in kwargs:
            del kwargs['schema']
        super(GeoAreaField, self).__init__(IGeoArea, **kwargs)



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from colander import MappingSchema, Decimal, Integer, SchemaNode, String

__docformat__ = 'restructuredtext'

from pyams_utils.rest import BaseResponseSchema


class Coordinate(MappingSchema):
    """Coordinate field"""
    latitude = SchemaNode(Decimal(),
                          description="Latitude")
    longitude = SchemaNode(Decimal(),
                           description="Longitude")
    
    
class SRID(Integer):
    """Spatial reference system identifier"""
    
    
class GeoPoint(MappingSchema):
    """Geographic point schema"""
    
    
class Area(MappingSchema):
    """Geographic rectangular area schema"""
    x1 = SchemaNode(Decimal(),
                    description="Upper-left point longitude coordinate")
    y1 = SchemaNode(Decimal(),
                    description="Upper-left point latitude coordinate")
    x2 = SchemaNode(Decimal(),
                    description="Lower-right point longitude coordinate")
    y2 = SchemaNode(Decimal(),
                    description="Lower-right point latitude coordinate")


class GeoPointTransformInfo(MappingSchema):
    """Geographic point transformation schema"""
    point = Coordinate(description="Point coordinates")
    from_srid = SchemaNode(SRID(),
                           description="Source SRID")
    to_srid = SchemaNode(SRID(),
                         description="Target SRID")
    
    
class GeoPointTransformRequest(MappingSchema):
    """Geographic point transformation request"""
    body = GeoPointTransformInfo(description="Point transformation request")
    
    
class GeoPointTransformResult(BaseResponseSchema):
    """Geographic point transformation result"""
    point = Coordinate()
    srid = SchemaNode(SRID(),
                      description="Coordinates SRID")
    
    
class GeoPointTransformResponse(MappingSchema):
    """Geographic point transformation response"""
    body = GeoPointTransformResult()
    
    
class GeoAreaTransformInfo(MappingSchema):
    """Geographic area transformation schema"""
    area = Area(description="Area coordinates")
    from_srid = SchemaNode(SRID(),
                           description="Source SRID")
    to_srid = SchemaNode(SRID(),
                         description="Target SRID")


class GeoAreaTransformRequest(MappingSchema):
    """Geographic area transformation request"""
    body = GeoAreaTransformInfo(description="Area transformation request")
    
    
class GeoAreaTransformResult(BaseResponseSchema):
    """Geographic area transformation result"""
    area = Area()
    srid = SchemaNode(SRID(),
                      description="Coordinates SRID")
    
    
class GeoAreaTransformResponse(MappingSchema):
    """Geographic area transformation response"""
    body = GeoAreaTransformResult()



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.zmi.interfaces module

This module defines interfaces used for PyAMS_gis management interface.
"""

from zope.interface import Interface

from pyams_skin.interfaces.view import IModalAddForm
from pyams_table.interfaces import ITable
from pyams_viewlet.interfaces import IViewletManager
from pyams_zmi.interfaces.viewlet import INavigationMenuItem

__docformat__ = 'restructuredtext'


class IMapManagerMenu(INavigationMenuItem):
    """Maps manager menu interface"""


class IMapManagerLayersTable(ITable):
    """Maps manager layers table marker interface"""


class IMapManagerLayersView(Interface):
    """Maps manager layers view interface"""


class IMapHeaderViewletManager(IViewletManager):
    """Map header viewlet manager marker interface"""


class IMapLayerAddForm(IModalAddForm):
    """Map layer add form marker interface"""



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_gis.zmi.interfaces import IMapHeaderViewletManager
from pyams_layer.interfaces import IFormLayer
from pyams_viewlet.manager import WeightOrderedViewletManager, viewletmanager_config


@viewletmanager_config(name='pyams_gis.map.header',
                       layer=IFormLayer, view=Interface,
                       provides=IMapHeaderViewletManager)
class MapHeaderViewletManager(WeightOrderedViewletManager):
    """Map header viewlet manager"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.zmi module

This module defines main maps manager layers view.
"""

from pyramid.decorator import reify
from pyramid.view import view_config
from zope.interface import Interface, implementer

from pyams_gis.interfaces import MANAGE_MAPS_PERMISSION
from pyams_gis.interfaces.utility import IMapManager
from pyams_gis.zmi.interfaces import IMapManagerLayersTable, IMapManagerLayersView, IMapManagerMenu
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IControlPanelMenu
from pyams_zmi.table import IconColumn, NameColumn, Table, TableAdminView, TableElementEditor, TrashColumn
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_gis import _


@adapter_config(required=(IMapManager, IAdminLayer, Interface),
                provides=ITableElementEditor)
class MapManagerTableElementEditor(TableElementEditor):
    """Maps manager table element editor"""
    
    view_name = 'admin#map-layers.html'
    modal_target = False
    
    def __new__(cls, context, request, view):
        if not request.has_permission(MANAGE_MAPS_PERMISSION, context=context):
            return None
        return TableElementEditor.__new__(cls)
    
    
@viewlet_config(name='maps.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IControlPanelMenu, weight=60,
                permission=MANAGE_MAPS_PERMISSION)
class MapManagerMenu(NavigationMenuItem):
    """Maps manager menu"""
    
    label = _("map-manager-menu", default="Maps manager")
    icon_class = 'fas fa-map-marker'
    
    def __new__(cls, context, request, view, manager):
        manager = query_utility(IMapManager)
        if (manager is None) or not manager.show_home_menu:
            return None
        return NavigationMenuItem.__new__(cls)
    
    def get_href(self):
        """Menu URL getter"""
        manager = get_utility(IMapManager)
        return absolute_url(manager, self.request, 'admin')


@viewletmanager_config(name='map-layers.menu',
                       context=IMapManager, layer=IAdminLayer,
                       manager=IControlPanelMenu, weight=60,
                       permission=MANAGE_MAPS_PERMISSION,
                       provides=IMapManagerMenu)
class MapManagerLayersMenu(NavigationMenuItem):
    """Maps manager layers menu"""
    
    label = _("Maps manager")
    icon_class = 'fas fa-map-marker'
    
    href = '#map-layers.html'


@pagelet_config(name='map-layers.html',
                context=IMapManager, layer=IPyAMSLayer,
                permission=MANAGE_MAPS_PERMISSION)
@implementer(IMapManagerLayersView)
class MapManagerLayersView(TableAdminView):
    """Map manager layers view"""

    title = _("Maps manager layers")
    table_class = IMapManagerLayersTable
    table_label = _("List of maps layers")

    @property
    def back_url(self):
        """Form back URL getter"""
        return absolute_url(self.request.root, self.request, 'admin#utilities.html')

    back_url_target = None


@factory_config(IMapManagerLayersTable)
class MapManagerLayersTable(Table):
    """Maps manager layers table"""
    
    display_if_empty = True
    
    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        manager = get_utility(IMapManager)
        attributes['table'].update({
            'data-ams-location': absolute_url(manager, self.request),
            'data-ams-order': '1,asc'
        })
        return attributes


@adapter_config(required=(IMapManager, IAdminLayer, IMapManagerLayersTable),
                provides=IValues)
class MapManagerLayersValues(ContextRequestViewAdapter):
    """Maps manager layers table values"""

    @property
    def values(self):
        """Maps manager layers table values getter"""
        yield from self.context.values()


@adapter_config(name='icon',
                required=(IMapManager, IAdminLayer, IMapManagerLayersTable),
                provides=IColumn)
class MapManagerLayersIconColumn(IconColumn):
    """Maps manager layers icon column"""

    weight = 1

    def get_icon_class(self, item):
        """Icon class getter"""
        return item.layer_icon

    def get_icon_hint(self, item):
        """Icon hint getter"""
        translate = self.request.localizer.translate
        return translate(item.layer_type)


@adapter_config(name='name',
                required=(IMapManager, IAdminLayer, IMapManagerLayersTable),
                provides=IColumn)
class MapManagerLayersNameColumn(NameColumn):
    """Maps manager layers name column"""


@adapter_config(name='trash',
                required=(IMapManager, IAdminLayer, IMapManagerLayersTable),
                provides=IColumn)
class MapManagerLayersTrashColumn(TrashColumn):
    """Maps manager layers trash column"""

    permission = MANAGE_MAPS_PERMISSION


@view_config(name='delete-element.json',
             context=IMapManager, request_type=IPyAMSLayer,
             permission=MANAGE_MAPS_PERMISSION, renderer='json', xhr=True)
def delete_map_layer(request):
    """Delete map layer"""
    return delete_map_layer(request)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.zmi.layer module

This module defines map layers management components.
"""

from pyramid.events import subscriber
from zope.interface import Interface, Invalid, implementer

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_gis.interfaces import MANAGE_MAPS_PERMISSION
from pyams_gis.interfaces.layer import IEsriFeatureMapLayer, IGeoportalMapLayer, IGoogleMapLayer, IMapLayer, \
    ITileMapLayer, IWMSMapLayer
from pyams_gis.interfaces.utility import IMapManager
from pyams_gis.layer import EsriFeatureMapLayer, GeoportalMapLayer, GoogleMapLayer, TileMapLayer, WMSMapLayer
from pyams_gis.zmi.interfaces import IMapManagerLayersTable, IMapLayerAddForm
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.event import get_json_table_row_add_callback, get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_gis import _


class MapLayerAddMenu(MenuItem):
    """Map layer add menu"""

    modal_target = True

    def get_href(self):
        manager = get_utility(IMapManager)
        return absolute_url(manager, self.request, self.href)


@implementer(IMapLayerAddForm)
class MapLayerAddForm(AdminModalAddForm):
    """Map layer add form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("New layer: {}")).format(translate(self.content_label))

    legend = _("New layer properties")
    content_factory = None
    content_label = MISSING_INFO

    @property
    def fields(self):
        return Fields(self.content_factory).omit('__name__', '__parent__')

    def add(self, obj):
        self.context[obj.name] = obj


@subscriber(IDataExtractedEvent, form_selector=IMapLayerAddForm)
def handle_new_map_layer_data(event):
    """Handle new layer data extraction"""
    manager = query_utility(IMapManager)
    name = event.data.get('name')
    if name in manager:
        event.form.widgets.errors += (Invalid(_("Specified layer name already exists!")))


@adapter_config(required=(IMapManager, IAdminLayer, IMapLayerAddForm),
                provides=IAJAXFormRenderer)
class MapLayerAddFormRenderer(ContextRequestViewAdapter):
    """Map layer add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:  # WARNING: creating an empty container will return a "false" value!
            return None
        manager = get_utility(IMapManager)
        return {
            'callbacks': [
                get_json_table_row_add_callback(manager, self.request,
                                                IMapManagerLayersTable, changes)
            ]
        }


@adapter_config(required=(IMapLayer, IPyAMSLayer),
                provides=IObjectLabel)
def map_layer_label(context, request):
    """Map layer label adapter"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(IMapLayer, IAdminLayer, Interface),
                provides=ITableElementEditor)
class MapLayerEditor(TableElementEditor):
    """Map layer editor adapter"""


@adapter_config(required=IMapLayer,
                provides=IViewContextPermissionChecker)
class MapLayerPermissionChecker(ContextAdapter):
    """Map layer permission checker"""

    edit_permission = MANAGE_MAPS_PERMISSION


class MapLayerPropertiesEditForm(AdminModalEditForm):
    """Map layer properties edit form"""

    legend = _("Layer properties")
    content_factory = None
    content_label = MISSING_INFO

    @property
    def fields(self):
        return Fields(self.content_factory).omit('__name__', '__parent__')

    def update_widgets(self, prefix=None):
        super().update_widgets()
        name = self.widgets.get('name')
        if name is not None:
            name.mode = DISPLAY_MODE


@adapter_config(required=(IMapLayer, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def map_layer_edit_form_title(context, request, form):
    """Map layer edit form title"""
    translate = request.localizer.translate
    manager = get_utility(IMapManager)
    layer = get_parent(context, IMapLayer)
    return TITLE_SPAN_BREAK.format(
        get_object_label(manager, request, form),
        translate(_("Layer: {} ({})")).format(get_object_label(layer, request, form),
                                              translate(layer.layer_type)))


@adapter_config(required=(IMapLayer, IAdminLayer, IModalEditForm),
                provides=IAJAXFormRenderer)
class MapLayerPropertiesAJAXRenderer(ContextRequestViewAdapter):
    """Map layer properties AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        manager = get_utility(IMapManager)  # pylint: disable=invalid-name
        return get_json_table_row_refresh_callback(manager, self.request,
                                                   IMapManagerLayersTable, self.context)


#
# Tile layers components
#

@viewlet_config(name='add-tile-layer.menu',
                context=IMapManager, layer=IAdminLayer, view=IMapManagerLayersTable,
                manager=IContextAddingsViewletManager, weight=10,
                permission=MANAGE_MAPS_PERMISSION)
class TileLayerAddMenu(MapLayerAddMenu):
    """Tile layer add menu"""

    label = _("Add tile layer...")
    href = 'add-tile-layer.html'
    icon_class = TileMapLayer.layer_icon


class TileLayerMixinForm:
    """Tile layer mixin form"""

    content_factory = ITileMapLayer
    content_label = TileMapLayer.layer_type


@ajax_form_config(name='add-tile-layer.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class TileLayerAddForm(TileLayerMixinForm, MapLayerAddForm):
    """Tile layer add form"""


@ajax_form_config(name='properties.html',
                  context=ITileMapLayer, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class TileLayerPropertiesEditForm(TileLayerMixinForm, MapLayerPropertiesEditForm):
    """Tile layer properties edit form"""


#
# WMS layers components
#

@viewlet_config(name='add-wms-layer.menu',
                context=IMapManager, layer=IAdminLayer, view=IMapManagerLayersTable,
                manager=IContextAddingsViewletManager, weight=20,
                permission=MANAGE_MAPS_PERMISSION)
class WMSLayerAddMenu(MapLayerAddMenu):
    """WMS layer add menu"""

    label = _("Add WMS layer...")
    href = 'add-wms-layer.html'
    icon_class = WMSMapLayer.layer_icon


class WMSLayerMixinForm:
    """WMS layer mixin form"""

    content_factory = IWMSMapLayer
    content_label = WMSMapLayer.layer_type


@ajax_form_config(name='add-wms-layer.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class WMSLayerAddForm(WMSLayerMixinForm, MapLayerAddForm):
    """WMS layer add form"""


@ajax_form_config(name='properties.html',
                  context=IWMSMapLayer, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class WMSLayerPropertiesEditForm(WMSLayerMixinForm, MapLayerPropertiesEditForm):
    """WMS layer properties edit form"""


#
# Geoportal layers components
#

@viewlet_config(name='add-geoportal-layer.menu',
                context=IMapManager, layer=IAdminLayer, view=IMapManagerLayersTable,
                manager=IContextAddingsViewletManager, weight=30,
                permission=MANAGE_MAPS_PERMISSION)
class GeoportalLayerAddMenu(MapLayerAddMenu):
    """Geoportal layer add menu"""

    label = _("Add Geoportal layer...")
    href = 'add-geoportal-layer.html'
    icon_class = GeoportalMapLayer.layer_icon


class GeoportalLayerMixinForm:
    """Geoportal layer mixin form"""

    content_factory = IGeoportalMapLayer
    content_label = GeoportalMapLayer.layer_type


@ajax_form_config(name='add-geoportal-layer.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class GeoportalLayerAddForm(GeoportalLayerMixinForm, MapLayerAddForm):
    """Geoportal layer add form"""


@ajax_form_config(name='properties.html',
                  context=IGeoportalMapLayer, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class GeoportalLayerPropertiesEditForm(GeoportalLayerMixinForm, MapLayerPropertiesEditForm):
    """Geoportal layer properties edit form"""


#
# ESRI feature layers components
#

@viewlet_config(name='add-esri-feature-layer.menu',
                context=IMapManager, layer=IAdminLayer, view=IMapManagerLayersTable,
                manager=IContextAddingsViewletManager, weight=40,
                permission=MANAGE_MAPS_PERMISSION)
class EsriFeatureLayerAddMenu(MapLayerAddMenu):
    """ESRI feature layer add menu"""

    label = _("Add ESRI feature layer...")
    href = 'add-esri-feature-layer.html'
    icon_class = EsriFeatureMapLayer.layer_icon


class EsriFeatureLayerMixinForm:
    """ESRI feature layer mixin form"""

    content_factory = IEsriFeatureMapLayer
    content_label = EsriFeatureMapLayer.layer_type


@ajax_form_config(name='add-esri-feature-layer.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class EsriFeatureLayerAddForm(EsriFeatureLayerMixinForm, MapLayerAddForm):
    """ESRI feature layer add form"""


@ajax_form_config(name='properties.html',
                  context=IEsriFeatureMapLayer, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class EsriFeatureLayerPropertiesEditForm(EsriFeatureLayerMixinForm, MapLayerPropertiesEditForm):
    """ESRI feature layer properties edit form"""


#
# Google layers components
#

@viewlet_config(name='add-google-layer.menu',
                context=IMapManager, layer=IAdminLayer, view=IMapManagerLayersTable,
                manager=IContextAddingsViewletManager, weight=50,
                permission=MANAGE_MAPS_PERMISSION)
class GoogleLayerAddMenu(MapLayerAddMenu):
    """Google layer add menu"""

    label = _("Add Google layer...")
    href = 'add-google-layer.html'
    icon_class = GoogleMapLayer.layer_icon


class GoogleLayerMixinForm:
    """Google layer mixin form"""

    content_factory = IGoogleMapLayer
    content_label = GoogleMapLayer.layer_type


@ajax_form_config(name='add-google-layer.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class GoogleLayerAddForm(GoogleLayerMixinForm, MapLayerAddForm):
    """Google layer add form"""


@ajax_form_config(name='properties.html',
                  context=IGoogleMapLayer, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class GoogleLayerPropertiesEditForm(GoogleLayerMixinForm, MapLayerPropertiesEditForm):
    """Google layer properties edit form"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

try:
    from osgeo.osr import SpatialReference, CoordinateTransformation
    have_gdal = True
except ImportError:
    have_gdal = False

from decimal import Decimal

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_gis.interfaces import CRS, IGeoPoint, IGeoPointZ
from pyams_utils.factory import factory_config
from pyams_utils.list import is_not_none

__docformat__ = 'restructuredtext'


@factory_config(IGeoPoint)
class GeoPoint(Persistent):
    """GeoPoint attribute object"""

    longitude = FieldProperty(IGeoPoint['longitude'])
    latitude = FieldProperty(IGeoPoint['latitude'])
    projection = FieldProperty(IGeoPoint['projection'])

    def __init__(self, data=None, **kwargs):
        super().__init__()
        if 'longitude' in kwargs:
            self.longitude = Decimal(kwargs['longitude'])
        if 'latitude' in kwargs:
            self.latitude = Decimal(kwargs['latitude'])
        if 'projection' in kwargs:
            self.projection = kwargs['projection']

    def __bool__(self):
        return len(tuple(map(is_not_none, (self.longitude, self.latitude)))) == 2

    def get_coordinates(self, projection=CRS.WGS84.value):
        if projection == self.projection:
            return self.longitude, self.latitude
        if (not have_gdal) or not self:
            return None, None
        source = SpatialReference()
        source.ImportFromEPSG(self.projection)
        destination = SpatialReference()
        destination.ImportFromEPSG(projection)
        transformation = CoordinateTransformation(source, destination)
        return transformation.TransformPoint(float(self.longitude), float(self.latitude))[0:2]

    @property
    def wgs_coordinates(self):
        return self.get_coordinates(CRS.WGS84.value)

    def to_json(self):
        if not self:
            return None
        return {
            'lon': float(self.longitude),
            'lat': float(self.latitude),
            'crs': float(self.projection)
        }


@factory_config(IGeoPointZ)
class GeoPointZ(GeoPoint):
    """GeoPointZ attribute object"""

    altitude = FieldProperty(IGeoPointZ['altitude'])

    def __init__(self, data=None, **kwargs):
        super().__init__(data, **kwargs)
        if 'altitude' in kwargs:
            self.altitude = Decimal(kwargs['altitude'])

    def to_json(self):
        result = super().to_json()
        if result:
            result.update({
                'alt': float(self.altitude)
            })
        return result



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.interfaces.configuration module

This module defines interface of map configuration.
"""

from zope.interface import Interface
from zope.schema import Bool, Choice, Int, List

from pyams_gis.interfaces import CRS, LAYER_CRS, LAYER_CRS_VOCABULARY
from pyams_gis.interfaces.layer import PYAMS_GIS_LAYERS_VOCABULARY
from pyams_gis.schema import GeoAreaField, GeoPointField

__docformat__ = 'restructuredtext'

from pyams_gis import _


MAP_CONFIGURATION_KEY = 'pyams_gis.configuration'


class IMapConfiguration(Interface):
    """Map configuration interface"""

    crs = Choice(title=_("CRS"),
                 description=_("Coordinates reference system to use for the map"),
                 vocabulary=LAYER_CRS_VOCABULARY,
                 default=LAYER_CRS[CRS.WGS84WM.value],
                 required=True)

    layers = List(title=_("Layers list"),
                  description=_("List of available layers displayed into this map"),
                  value_type=Choice(vocabulary=PYAMS_GIS_LAYERS_VOCABULARY),
                  required=False)

    auto_adjust = Bool(title=_("Adjust bounds to markers layer"),
                       description=_("If 'yes', map area will be automatically adjusted "
                                     "to markers layer(s), if any"),
                       required=True,
                       default=True)

    zoom_level = Int(title=_("Initial zoom level"),
                     description=_("Zoom level at which to display map, if auto-adjust is "
                                   "disabled or if there is only one marker"),
                     min=0,
                     max=18,
                     default=11,
                     required=False)

    initial_center = GeoPointField(title=_("Initial center"),
                                   description=_("Initial map location center"),
                                   required=False)

    initial_bounds = GeoAreaField(title=_("Initial bounds"),
                                  description=_("Initial map location bounds, if auto-adjust or "
                                                "initial center are disabled"),
                                  required=False)

    attribution_control = Bool(title=_("Attribution control?"),
                               description=_("If 'yes', an attribution control is added to map"),
                               required=True,
                               default=True)

    zoom_control = Bool(title=_("Zoom control?"),
                        description=_("If 'yes', a zoom control is added to map"),
                        required=True,
                        default=True)

    fullscreen_control = Bool(title=_("Fullscreen control?"),
                              description=_("If 'yes', a fullscreen control is added to map"),
                              required=True,
                              default=False)

    layer_control = Bool(title=_("Layers control?"),
                         description=_("If 'yes', a layer selection control is added to map"),
                         required=True,
                         default=False)

    keyboard = Bool(title=_("Keyboard navigation?"),
                    description=_("If 'yes', makes the map focusable and allows users to "
                                  "navigate with keyboard arrows and +/- keys; this option "
                                  "will not be activated on mobile devices"),
                    required=True,
                    default=True)

    scroll_wheel_zoom = Bool(title=_("Scroll wheel zoom?"),
                             description=_("If 'yes', the map can be zoomed using the mouse "
                                           "wheel; this should be avoided to get a good "
                                           "responsive behaviour and handle gestures"),
                             required=True,
                             default=False)

    def get_configuration(self):
        """Get map layers configuration"""


class IMapConfigurationTarget(Interface):
    """Map configuration target marker interface"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.zmi.configuration module

This module define PyAMS_gis default maps configuration management components.
"""

from pyramid.view import view_config
from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent
from pyams_gis.interfaces import MANAGE_MAPS_PERMISSION
from pyams_gis.interfaces.configuration import IMapConfiguration
from pyams_gis.interfaces.utility import IMapManager
from pyams_gis.zmi.interfaces import IMapManagerMenu
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_gis import _


@view_config(name='get-map-configuration.json',
             context=Interface, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def get_map_configuration(request):
    """Get map configuration in JSON format"""
    manager = query_utility(IMapManager)
    if manager is not None:
        configuration = IMapConfiguration(manager)
        if configuration is not None:
            return configuration.get_configuration()


@viewlet_config(name='map-manager-configuration.menu',
                context=IMapManager, layer=IAdminLayer,
                manager=IMapManagerMenu, weight=10,
                permission=MANAGE_MAPS_PERMISSION)
class MapManagerConfigurationMenu(NavigationMenuItem):
    """Map manager configuration menu"""

    label = _("Default configuration")
    href = '#configuration.html'


@ajax_form_config(name='configuration.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class MapManagerConfigurationEditForm(AdminEditForm):
    """Map manager configuration edit form"""

    title = _("Map manager default configuration")
    legend = _("Default map configuration")

    fields = Fields(IMapConfiguration)


@adapter_config(required=(IMapManager, IAdminLayer, MapManagerConfigurationEditForm),
                provides=IFormContent)
def map_manager_configuration_content(context, request, form):
    """Map manager configuration content"""
    return IMapConfiguration(context)
